package it.coderunner.spring.data.controller;

/**
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import org.json.simple.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.context.request.WebRequest;

import it.coderunner.spring.data.model.City;
import it.coderunner.spring.data.service.CityService;

@SuppressWarnings("unchecked")
@Controller
public class CityController {

	@Autowired
	private CityService cityService;

	@GetMapping("/save/city/{name}/{voivodeship}")
	public @ResponseBody JSONObject saveCity(@PathVariable String name, @PathVariable String voivodeship) {
		City city = cityService.save(new City(name, voivodeship));
		JSONObject obj = new JSONObject();
		
		obj.put("success", city != null);
		obj.put("message",  city != null ? "Pomyślnie zapisano informacje o mieście " + name : "Wystąpił problem z zapisem");
		obj.put("tableIndex", city != null ? city.getId() : -1);
		return obj;		
	}

	@GetMapping("/get/city/{name}/{voivodeship}")
	public @ResponseBody JSONObject getByNameAndVoivodeship(@PathVariable String name,
			@PathVariable String voivodeship) {
		City city = cityService.findByNameAndVoivodeshipAllIgnoringCase(name, voivodeship);
		JSONObject obj = new JSONObject();
		JSONObject contentObj = new JSONObject();
		if (city != null) {
			contentObj = createJSON(city);
		} 
		obj.put("content", contentObj);
		obj.put("success", city != null);
		obj.put("message",  city != null ? "" : "Nie znaleziono miasta");
		return obj;
	}

	@GetMapping("/get/all/{page}")
	public @ResponseBody Page<City> getTop10(@PathVariable Integer page) {
		return cityService.findAll(new PageRequest(page, 10));
	}

	@PostMapping("/update/city/all")
	public @ResponseBody JSONObject updateCity(WebRequest request) {
		Long id = Long.valueOf(request.getParameter("id"));
		City city = cityService.findOne(id);
		Boolean responseStatus = city != null;
		String response = responseStatus ? "Pomyślnie zapisano informacje o mieście " + city.getName() : "Nie znaleziono miasta";
		
		JSONObject obj = new JSONObject();
		if (responseStatus)
		try {
			city.setVoivodeship(request.getParameter("voivodeship"));
			city.setBirthdate(request.getParameter("birthdate"));
			city.setNote(request.getParameter("note"));
			cityService.save(city);
		} catch(Exception ex) {
			response = ex.getLocalizedMessage();
			responseStatus = false;
		}

		obj.put("success", responseStatus);
		obj.put("message",  response);		
		obj.put("tableIndex", responseStatus ? city.getId() : -1);
		return obj;
	}
	
	@GetMapping("/delete/city/{id}")
	public @ResponseBody JSONObject deleteCity(@PathVariable Long id) {
		Boolean responseStatus = true;
		String response = "Pomyślnie usunięto rekord #" + id;
		JSONObject obj = new JSONObject();
		try {
			cityService.delete(id);
		} catch(Exception ex) {
			response = ex.getLocalizedMessage();
			responseStatus = false;
		}

		obj.put("success", responseStatus);
		obj.put("message",  response);			
		return obj;
	}

	private JSONObject createJSON(City city) {
		JSONObject content = new JSONObject();
		content.put("id", city.getId());
		content.put("name", city.getName());
		content.put("voivodeship", city.getVoivodeship());
		content.put("birthdate", city.getBirthdate());
		content.put("note", city.getNote());
		return content;
	}
	
}
